package com.example.bondoyap.ui.transactions

import android.content.Context
import android.content.Intent
import android.location.Address
import android.location.Geocoder
import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import android.widget.Toast
import androidx.cardview.widget.CardView
import androidx.core.content.ContextCompat.startActivity
import androidx.navigation.Navigation
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.example.bondoyap.R
import com.example.bondoyap.ui.transactions.data.Transactions
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.io.IOException
import java.util.Locale

class TransactionsListAdapter(private val context: Context) :
    ListAdapter<Transactions, TransactionsViewHolder>(TransactionsDiffCallback()) {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): TransactionsViewHolder {
        return TransactionsViewHolder.create(parent, context)
    }

    override fun onBindViewHolder(holder: TransactionsViewHolder, position: Int) {
        val currentTransaction = getItem(position)
        holder.bind(currentTransaction)
    }

}

class TransactionsViewHolder(itemView: View, private val context: Context) : RecyclerView.ViewHolder(itemView) {
    private val cardView: CardView = itemView.findViewById(R.id.cardViewTransaction)
    private val transactionTitle: TextView = itemView.findViewById(R.id.transactionTitle)
    private val transactionAmount: TextView = itemView.findViewById(R.id.transactionAmount)
    private val transactionCategory: TextView = itemView.findViewById(R.id.transactionCategory)
    private val transactionDate: TextView = itemView.findViewById(R.id.transactionDate)
    private val transactionLocation: TextView = itemView.findViewById(R.id.transactionLocation)
    private val geocoder: Geocoder = Geocoder(context, Locale.getDefault())

    fun bind(transaction: Transactions) {
        val maxAmountLength = 12
        val maxTitleLength = 16
        val maxLocationLength = 9

        val amountText = if (transaction.nominal.toBigDecimal().toString().length > maxAmountLength) {
            "IDR " + transaction.nominal.toBigDecimal().toString().substring(0, maxAmountLength) + "..."
        } else {
            "IDR " + transaction.nominal.toBigDecimal().toString()
        }

        val titleText = if (transaction.judul.length > maxTitleLength) {
            transaction.judul.substring(0, maxTitleLength) + "..."
        } else {
            transaction.judul
        }

        transactionTitle.text = titleText
        transactionAmount.text = amountText
        transactionCategory.text = when (transaction.isPemasukan) {
            true -> "Pemasukan"
            else -> "Pengeluaran"
        }

        transactionDate.text = transaction.tanggal

        if (transaction.latitude.isNotEmpty() && transaction.longitude.isNotEmpty()) {
            CoroutineScope(Dispatchers.IO).launch {
                try {
                    val addresses: List<Address> = geocoder.getFromLocation(
                        transaction.latitude.toDouble(),
                        transaction.longitude.toDouble(),
                        1
                    ) ?: emptyList()
                    if (addresses.isNotEmpty()) {
                        val locationName = addresses[0].getAddressLine(0)
                        Handler(Looper.getMainLooper()).post {
                            transactionLocation.text = if (locationName.length > maxLocationLength) {
                                locationName.substring(0, maxLocationLength) + "..."
                            } else {
                                locationName
                            }
                        }
                    }
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            }
        } else {
            transactionLocation.text = "Unavailable"
        }

        transactionLocation.setOnClickListener {
            if (transactionLocation.text != "Unavailable") {
                val mapUri = Uri.parse("https://maps.google.com/maps/search/?api=1&query=${transaction.latitude},${transaction.longitude}")
                val intent = Intent(Intent.ACTION_VIEW, mapUri)
                intent.setPackage("com.google.android.apps.maps")
                if (intent.resolveActivity(context.packageManager) != null) {
                    startActivity(context, intent, null)
                } else {
                    val webIntent = Intent(Intent.ACTION_VIEW, Uri.parse("https://www.google.com/maps/search/?api=1&query=${transaction.latitude},${transaction.longitude}"))
                    if (webIntent.resolveActivity(context.packageManager) != null) {
                        startActivity(context, webIntent, null)
                    } else {
                        Toast.makeText(context, "Tidak ada app yang dapat menghandle maps", Toast.LENGTH_SHORT).show()
                    }
                }
            }
        }

        cardView.setOnClickListener {
            val bundle: Bundle = Bundle()
            bundle.putInt("transaction_id", transaction.id)
            Navigation.findNavController(itemView).navigate(R.id.navigation_edit_transactions, bundle)
        }
    }

    companion object {
        fun create(parent: ViewGroup, context: Context): TransactionsViewHolder {
            val view: View = LayoutInflater.from(parent.context)
                .inflate(R.layout.recyclerview_transactions, parent, false)
            return TransactionsViewHolder(view, context)
        }
    }
}

class TransactionsDiffCallback : DiffUtil.ItemCallback<Transactions>() {
    override fun areItemsTheSame(oldItem: Transactions, newItem: Transactions): Boolean {
        return oldItem.id == newItem.id
    }

    override fun areContentsTheSame(oldItem: Transactions, newItem: Transactions): Boolean {
        return oldItem == newItem
    }
}
